BlendState SrcAlphaBlendingAdd
{
    BlendEnable[0] = TRUE;
    SrcBlend = SRC_ALPHA;
    DestBlend = INV_SRC_ALPHA;
    BlendOp = ADD;
    SrcBlendAlpha = ZERO;
    DestBlendAlpha = ZERO;
    BlendOpAlpha = ADD;
    RenderTargetWriteMask[0] = 0x0F;
};

DepthStencilState DisableDepth
{
    DepthEnable    = FALSE;
    DepthWriteMask = ZERO;
};

Texture2D mTexture : TEXTURE0;

SamplerState BilinearSampler
{
    Filter   = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
};


cbuffer olBuffer
{
	float mDepth;
}

float   mAlpha;

struct VS_Output
{
    float4 Pos : SV_POSITION;
    float3 Tex : TEXCOORD0;
};


VS_Output VS_DrawOverlay(in float4 position: POSITION)
{
    VS_Output OUT;

    OUT = (VS_Output)0;

    OUT.Pos    = float4(position.x, position.y, mDepth, 1.0f);
    OUT.Tex.xy = position.zw;
    OUT.Tex.z  = 1.0;

    return OUT;
}


float4 PS_DrawOverlay(in VS_Output In) : SV_TARGET0
{
    float3 c = mTexture.Sample(BilinearSampler, In.Tex.xy).rgb;

    return float4(c, c.x*mAlpha);
}


float4 PS_DrawOverlayWithoutAlphaMap(in VS_Output In) : SV_TARGET0
{
    float3 c = mTexture.Sample(BilinearSampler, In.Tex.xy).rgb;

    return float4(c, mAlpha);
}

//===================================================================
technique10 DrawOverlay
{
    pass p0
    {
        SetVertexShader  ( CompileShader( vs_4_0, VS_DrawOverlay() ) );
        SetGeometryShader( NULL );
        SetPixelShader   ( CompileShader( ps_4_0, PS_DrawOverlay() ) );

        SetDepthStencilState( DisableDepth, 0 );
        SetBlendState( SrcAlphaBlendingAdd, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
    }
}
technique10 DrawOverlayWithoutAlphaMap
{
    pass p0
    {
        SetVertexShader  ( CompileShader( vs_4_0, VS_DrawOverlay() ) );
        SetGeometryShader( NULL );
        SetPixelShader   ( CompileShader( ps_4_0, PS_DrawOverlayWithoutAlphaMap() ) );

        SetDepthStencilState( DisableDepth, 0 );
        SetBlendState( SrcAlphaBlendingAdd, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
    }
}
