

bool bVertical = false;
float fStepSize = 1.0f;
Texture2D blurTexture;
shared cbuffer permanent
{
    float fTextureWidth;
    float fFilterWidth = 9.0f;
};

struct VSQuadOut
{
    float4 pos : SV_Position;
    float2 tex : TEXCOORD;
};

BlendState RGBABS
{
    RenderTargetWriteMask[0] = 0x0f;
    BlendEnable[0] = false;
};

RasterizerState noCullRS
{
    CullMode = None;
    FillMode = Solid;
    MultisampleEnable = false;
};

DepthStencilState zWriteNoStencilDS
{
    DepthEnable = true;
    DepthFunc = Less_Equal;
    DepthWriteMask = All;
    StencilEnable = false;
};

SamplerState FILT_LINEAR
{
    AddressU = Clamp;
    AddressV = Clamp;
    Filter = MIN_MAG_MIP_LINEAR;
};

float BoxFilterStart( float fWidth )
{
    return ( ( fWidth - 1.0f ) / 2.0f );
}

VSQuadOut VSFullscreenQuad( uint id : SV_VertexID )
{
    VSQuadOut output = (VSQuadOut)0.0f;
    output.tex = float2( (id << 1) & 2, id & 2 );
    output.pos = float4( output.tex * float2( 2.0f, -2.0f ) + float2( -1.0f, 1.0f), 0.0f, 1.0f );
    
    return output;
}

float4 PSFilter( VSQuadOut input ) : SV_Target
{
    float fStartOffset = BoxFilterStart( fFilterWidth );
    float2 fTexelOffset = float2( bVertical * ( fStepSize / fTextureWidth ), !bVertical * ( fStepSize / fTextureWidth ) );
    
    float2 fTexStart = input.tex - ( fStartOffset * fTexelOffset );
    float4 output = (float4)0.0f;
    
    for( int i = 0; i < fFilterWidth; ++i )
        output += blurTexture.Sample( FILT_LINEAR, float2( fTexStart + fTexelOffset * i) );
    
    return output / fFilterWidth;
}


technique10 BlurFilter
{
    pass p0
    {
        SetVertexShader( CompileShader( vs_4_0, VSFullscreenQuad() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PSFilter() ) );
    
        SetRasterizerState( noCullRS );
        SetBlendState( RGBABS, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xffffffff );
        SetDepthStencilState( zWriteNoStencilDS, 0xffffffff );
    }
}