BlendState SrcAlphaBlendingAdd
{
    BlendEnable[0] = TRUE;
    SrcBlend = SRC_ALPHA;
    DestBlend = INV_SRC_ALPHA;
    BlendOp = ADD;
    SrcBlendAlpha = ZERO;
    DestBlendAlpha = ZERO;
    BlendOpAlpha = ADD;
    RenderTargetWriteMask[0] = 0x0F;
};

DepthStencilState DisableDepth
{
    DepthEnable    = FALSE;
    DepthWriteMask = ZERO;
};

RasterizerState backCullRS
{
    CullMode = Back;
    FillMode = Solid;
    MultisampleEnable = false;
};

Texture3D   mVolumeTexture;
Texture2D   mBackFaceTexture;

SamplerState TexSamp
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
    AddressW = Clamp;
};


float4x4 mWorldViewProjection;    // World * View * Projection matrix

struct VS_Output
{
    float4 Pos : SV_POSITION;
    float3 Tex : TEXCOORD0;
    float4 SSPos : TEXCOORD1;
};


VS_Output VS_Render(	float4 position: POSITION,
								float3 texcoord : TEXCOORD0)
{
    VS_Output OUT;

    OUT = (VS_Output)0;

    OUT.Pos = mul(position, mWorldViewProjection);
    OUT.Tex = texcoord;
    OUT.SSPos = OUT.Pos;

    return OUT;
}


float4 PS_RenderBackFaces(in VS_Output In) : SV_TARGET0
{
    return float4(In.Tex, 1.0f);
}

float4 PS_RenderVolume(in VS_Output In) : SV_TARGET0
{
	int samples = 40;
    float3 entryPoint = In.Tex;
    float4 positionSS = In.SSPos;
    // Do the perspective divide
  	positionSS.xy /= positionSS.w;
  	// Convert screen space position from [-1,-1]->[1,1] to [0,0]->[1,1]
  	// Get this out for correct alhambra rendering:
  	//positionSS.xy *= 0.99f;
	positionSS.xy = positionSS.xy * 0.5 + 0.5;
	positionSS.y = 1.0f - positionSS.y;
	float3 exitPoint = mBackFaceTexture.Sample(TexSamp, positionSS.xy);
	
	float3 ray = exitPoint-entryPoint;
	
	if (length(ray)<0.05f)
		discard;
	
	float3 shortray = ray/samples;
	
	float4 color = float4(0.0f, 0.0f, 0.0f, 0.0f);
	
	float3 texcoords = entryPoint;
	
	
	for( int i=0; i<samples ; i++ )
    {
        float t;
        float4 coord = float4(texcoords, 0.02f);
        float4 sample = mVolumeTexture.SampleLevel(TexSamp, texcoords, 0);
        
        /*
        t = coord.a * (1.0-color.a);
        color.r += t * coord.r;
        color.g += t * coord.g;
        color.b += t * coord.b;
        
        color.a += t;
        */
        
        t = sample.a * (1.0-color.a);
        color.r += t * sample.r;
        color.g += t * 2.5 * sample.g;
        color.b += t * 3.5 * sample.b;
        
        color.a += t;
        
        texcoords += shortray;

        //if( color.a > 0.99 )
        //	break;
    }
    
    //if (color.a < 0.05f)
    //	color.a = 0.0f;
    
    return color;
}


//===================================================================
technique10 RenderBackfaces
{
    pass p0
    { 
        SetVertexShader  ( CompileShader( vs_4_0, VS_Render() ) );
        SetGeometryShader( NULL );
        SetPixelShader   ( CompileShader( ps_4_0, PS_RenderBackFaces() ) );
    	SetRasterizerState( backCullRS ); 

        SetBlendState( SrcAlphaBlendingAdd, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
    }
}

technique10 RenderVolume
{
    pass p0
    { 
        SetVertexShader  ( CompileShader( vs_4_0, VS_Render() ) );
        SetGeometryShader( NULL );
        SetPixelShader   ( CompileShader( ps_4_0, PS_RenderVolume() ) );
    	SetRasterizerState( backCullRS ); 

        SetBlendState( SrcAlphaBlendingAdd, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
    }
}
